@@ -12,6 +12,7 @@ Example:
 
 from .model_io import *
 from .model_info import *
+from .logging import *
 
 from .task_timecourse import *
 from .task_parameterestimation import *
@@ -19,12 +20,13 @@ from .task_steadystate import *
 from .task_scan import *
 from .task_optimization import *
 from .task_sensitivities import *
+from .task_mca import *
+from .task_lyapunov import *
+from .task_lna import *
 
-from .array_tools import *
 from .compartment_array_tools import *
 
 settings = {}
 
-from ._version import get_versions
-__version__ = get_versions()['version']
-del get_versions
+from . import _version
+__version__ = _version.get_versions()['version']
