@@ -66,7 +66,11 @@ def __build_result_from_ts(time_series, use_concentrations=True, use_sbml_id=Fal
                 concentrations[i, j] = time_series.getData(i, j)
 
     df = pandas.DataFrame(data=concentrations, columns=column_names)
-    df = df.set_index('Time')
+
+    if len(column_names) != len(set(column_names)):
+        logger.warning('Duplicate column names in time series consider using use_sbml_id=True, or running ensure_unique_names()')
+    else:
+        df = df.set_index('Time')
 
     return df
 
@@ -173,7 +177,7 @@ def run_time_course_with_output(output_selection, *args, **kwargs):
             logger.error("Error while running the simulation: " +
                          model_info.get_copasi_messages(num_messages_before))
 
-    task.restore()
+    task.restore(True)
     df = get_data_from_data_handler(dh, columns)
     model.removeInterface(dh)
 
@@ -222,6 +226,9 @@ def create_data_handler(output_selection, during=None, after=None, before=None,
     dh = COPASI.CDataHandler()
     columns = []
     for name in output_selection:
+        if isinstance(name, COPASI.CCommonName):
+            name = name.getString()
+            
         if name.startswith('CN='):
             obj = model.getObject(COPASI.CCommonName(name))
             if not obj:
@@ -233,8 +240,34 @@ def create_data_handler(output_selection, during=None, after=None, before=None,
             obj = model.findObjectByDisplayName(name)
 
             if not obj:
-                logger.warning('no object for name {0}'.format(name))
-                continue
+                # add some heuristics
+                found = False
+                tail = name.rfind(').')
+                if name.startswith('(') and tail != -1:
+                    obj = model.getModel().getReaction(name[1:tail])
+                    if obj:
+                        parameter = name[tail+2:]
+                        
+                        if parameter == "Flux":
+                            obj = obj.getFluxReference()
+                            found = True
+                        elif parameter == "ParticleFlux":
+                            obj = obj.getParticleFluxReference()
+                            found = True
+                        else:
+                            params = obj.getParameterObjects()
+                            for data_obj in params:
+                                if data_obj[0].getObjectName() == parameter:
+                                    obj = data_obj[0].getValueObject()
+                                    found = True
+                                    break
+                        
+                        if not found: 
+                            obj = None
+                
+                if not found:
+                    logger.warning('no object for name {0}'.format(name))
+                    continue
 
             if obj.getObjectType() != 'Reference':
                 obj = obj.getValueReference()
@@ -243,13 +276,13 @@ def create_data_handler(output_selection, during=None, after=None, before=None,
             columns.append(name)
 
         if during is None or (during is not None and name in during):
-            dh.addDuringName(COPASI.CRegisteredCommonName(cn))
+            dh.addDuringName(model_info._get_registered_common_name(cn, dm=model))
 
         if after and name in after:
-            dh.addAfterName(COPASI.CRegisteredCommonName(cn))
+            dh.addAfterName(model_info._get_registered_common_name(cn, dm=model))
 
         if before and name in before:
-            dh.addAfterName(COPASI.CRegisteredCommonName(cn))
+            dh.addBeforeName(model_info._get_registered_common_name(cn, dm=model))
     return dh, columns
 
 
@@ -381,6 +414,12 @@ def _setup_timecourse(args, kwargs):
             for val in vals:
                 new_vals += ' ' + str(val)
             vals = new_vals.strip()
+
+        # ensure that the output start time is correct adjusted: 
+        first_value = vals.split(' ')[0] if vals else None
+        if first_value:
+            problem.setOutputStartTime(float(first_value))
+
         problem.setValues(vals)
         problem.setUseValues(True)
     if 'use_values' in kwargs:
