@@ -5,6 +5,12 @@ import unittest
 import basico
 import COPASI
 
+try:
+    import libsbml
+    _have_libsbml = True
+except ImportError:
+    _have_libsbml = False
+
 
 class TestBasicoIO_Brus(unittest.TestCase):
 
@@ -14,6 +20,35 @@ class TestBasicoIO_Brus(unittest.TestCase):
         self.assertTrue(isinstance(self.dm, COPASI.CDataModel))
         self.assertTrue('The Brusselator' in basico.model_io.overview())
 
+    @unittest.skipIf(not _have_libsbml, "libsbml not available")
+    def test_to_python_ode_string(self):
+        code = basico.model_io.to_python_ode_string()
+        self.assertTrue(code is not None)
+        self.assertTrue(len(code) > 0)
+
+        # instantiate the code
+        ctx = {}
+        exec(code, ctx)
+        self.assertTrue('ode_function' in ctx)
+        self.assertTrue(callable(ctx['ode_function']))
+
+        # call the function with the initial values and parameters 
+        y0 = ctx['y0']
+        p = ctx['parameter_values']
+        result = ctx['ode_function'](y0, 0, p)
+        self.assertTrue(result is not None)
+        self.assertTrue(len(result) == len(y0))
+
+        # call odeint with the function
+        from scipy.integrate import odeint
+        import numpy as np
+        y0 = ctx['y0']
+        t = np.linspace(0, 10, 100)
+        result = odeint(ctx['ode_function'], y0, t, args=(ctx['parameter_values'],))
+        self.assertTrue(result is not None)
+        self.assertTrue(result.shape[0] == len(t))
+        self.assertTrue(result.shape[1] == len(y0))
+
     def test_get_species(self):
         species = basico.get_species('X')
         self.assertTrue(species.shape[0] == 1)
@@ -181,7 +216,7 @@ class TestBasicoIO_Brus(unittest.TestCase):
         mod = basico.get_current_model().getModel()
         assert (isinstance(mod, COPASI.CModel))
         mod.applyInitialValues()
-        data = basico.model_info._collect_data(cns=[
+        data = basico.model_info.collect_data(cns=[
             'CN=Root,Model=The Brusselator,Reference=Time',
             'CN=Root,Model=The Brusselator,Vector=Compartments[compartment],Reference=InitialVolume',
             'CN=Root,Model=The Brusselator,Vector=Compartments[compartment],Reference=Rate',
@@ -200,7 +235,7 @@ class TestBasicoIO_Brus(unittest.TestCase):
             'CN=Root,Model=The Brusselator,Vector=Reactions[R1],ParameterGroup=Parameters,Parameter=k1,Reference=Value'
         ])
 
-        data2 = basico.model_info._collect_data(names=[
+        data2 = basico.model_info.collect_data(names=[
             'Time',
             'Compartments[compartment].InitialVolume',
             'Compartments[compartment].Rate',
@@ -269,6 +304,33 @@ class TestBasicoIO_LM(unittest.TestCase):
         m = basico.get_reaction_mapping('R3')
         self.assertListEqual(m['product'], ['S', 'E', 'F'])
 
+        # add new reaction with hill kinetics
+        basico.add_reaction('R4', 'S -> E', function='Hill Cooperativity')
+        basico.add_parameter('hillcoeff')
+        basico.add_parameter('K_ER_1')
+        basico.add_parameter('v_ER_1')
+        basico.add_species('Ca_1')
+            
+        m1 = basico.get_reaction_mapping('R4')
+        basico.set_reaction_mapping('R4', {'h': 2.0, 'substrate': 'Ca_1', 'Shalve': 'K_ER_1', 'V': 'v_ER_1'})
+        m2 = basico.get_reaction_mapping('R4')
+        self.assertEqual(m2['substrate'], 'Ca_1')
+        self.assertEqual(m2['Shalve'], 'K_ER_1')
+        self.assertEqual(m2['V'], 'v_ER_1')
+        self.assertEqual(m2['h'], 2.0)
+        basico.set_reaction_mapping('R4', {'h': 2.0, 'substrate': 'Ca_1', 'Shalve': 0.1, 'V': 0.1})
+        m3 = basico.get_reaction_mapping('R4')
+        self.assertEqual(m3['Shalve'], 0.1)
+        self.assertEqual(m3['V'], 0.1)
+        basico.set_reaction_mapping('R4', {'substrate': 'Ca_1', 'Shalve': 'K_ER_1', 'V': 'v_ER_1', 'h': 'hillcoeff'})
+        m4 = basico.get_reaction_mapping('R4')
+        self.assertEqual(m4['Shalve'], 'K_ER_1')
+        self.assertEqual(m4['V'], 'v_ER_1')
+        self.assertEqual(m4['h'], 'hillcoeff')
+
+
+        
+
     @unittest.skipIf(sys.version_info < (3, 6, 0), 'This test requires assertLogs which is not available before')
     def test_invalid_expression(self):
         self.assertLogs(
@@ -305,6 +367,12 @@ class TestBasicoModelConstruction(unittest.TestCase):
 
     def test_species(self):
         basico.add_species('A', initial_concentration=10)
+
+        # set and get notes
+        basico.set_notes('Notes on the species A', name='A')
+        notes = basico.get_notes(name='A')
+        self.assertTrue('Notes on the species A' in notes)
+
         a = basico.as_dict(basico.get_species('A'))
         self.assertIsNotNone(a)
         self.assertAlmostEqual(a['initial_concentration'], 10)
@@ -339,6 +407,12 @@ class TestBasicoModelConstruction(unittest.TestCase):
 
     def test_compartment(self):
         basico.add_compartment('v',  initial_size=2)
+
+        # set and get notes
+        basico.set_notes('Notes on the compartment v', name='v')
+        notes = basico.get_notes(name='v')
+        self.assertTrue('Notes on the compartment v' in notes)
+
         v = basico.get_compartments('v', exact=True)
         self.assertIsNotNone(v)
         basico.set_compartment('v',  size=3)
@@ -350,9 +424,26 @@ class TestBasicoModelConstruction(unittest.TestCase):
         self.assertEqual(v['type'], 'assignment')
         self.assertEqual(v['expression'], 'Time')
         self.assertEqual(v['initial_expression'], '')
+        self.assertEqual(v['dimensionality'], 3)
+
+
+        # change dimensions to two
+        basico.set_compartment('v', exact=True, dimensionality=2)
+        v_df = basico.get_compartments('v')
+        v = basico.as_dict(v_df)
+        self.assertEqual(v['dimensionality'], 2)
+
+        basico.set_compartment('v', exact=True, dimensionality=v['dimensionality'])
+        basico.set_compartment('v', exact=True, dimensionality=v_df.iloc[0]['dimensionality'])
+
+        # apply all recorded changes
+        basico.set_compartment(exact=True, **v)
+        
+
         basico.remove_compartment('v')
         self.assertIsNone(basico.get_compartments('v', exact=True))
 
+
     def test_global_parameters(self):
         basico.add_parameter('p', initial_value=3)
         v = basico.get_parameters('p', exact=True)
@@ -368,6 +459,12 @@ class TestBasicoModelConstruction(unittest.TestCase):
 
     def test_functions(self):
         basico.add_function('fun', infix='v * S / (k + S)', mapping={'S': 'substrate'})
+
+        # set and get notes
+        basico.set_notes('Notes on the function fun', name='fun')
+        notes = basico.get_notes(name='fun')
+        self.assertTrue('Notes on the function fun' in notes)
+
         fun = basico.as_dict(basico.get_functions('fun', exact=True))
         self.assertIsNotNone(fun)
         self.assertEqual(fun['reversible'], False)
@@ -381,6 +478,12 @@ class TestBasicoModelConstruction(unittest.TestCase):
         basico.add_parameter('p0', initial_value=10)
         basico.add_event('e0', trigger='Time > 10', assignments=[('Values[p0]', 1)],
                          fire_at_initial_time=True, persistent=True, delay_calculation=False)
+        
+        # set and get notes
+        basico.set_notes('Notes on the event e0', name='e0')
+        notes = basico.get_notes(name='e0')
+        self.assertTrue('Notes on the event e0' in notes)
+
         e = basico.as_dict(basico.get_events('e0', exact=True))
         self.assertIsNotNone(e)
         self.assertEqual(e['trigger'], 'Time > 10')
@@ -395,6 +498,12 @@ class TestBasicoModelConstruction(unittest.TestCase):
 
     def test_reactions(self):
         basico.add_reaction('r0', 'A -> B')
+
+        # set and get notes
+        basico.set_notes('Notes on the reaction r0', name='r0')
+        notes = basico.get_notes(name='r0')
+        self.assertTrue('Notes on the reaction r0' in notes)
+
         r0 = basico.as_dict(basico.get_reactions('r0', exact=True))
         self.assertIsNotNone(r0)
         self.assertEqual(r0['scheme'], 'A -> B')
@@ -532,5 +641,60 @@ class TestBasicoModelConstruction(unittest.TestCase):
         basico.remove_datamodel(dm1)
         basico.remove_datamodel(dm2)
 
+    def test_expresssions(self):
+        dm = basico.new_model(name='Test Model');
+        basico.add_species('X');
+        basico.add_species('Y');
+        basico.add_parameter('a', initial_value=1);
+
+        basico.add_parameter('b', initial_value=1);
+        basico.set_parameters('b', exact=True, expression='{Values[a].InitialValue}/2', status='assignment')
+        params1 = basico.as_dict(basico.get_parameters(type='assignment'))
+        basico.set_parameters('b', exact=True, expression='Values[a].InitialValue/2', status='assignment')
+        params2 = basico.as_dict(basico.get_parameters(type='assignment'))
+
+        basico.add_parameter('c', initial_value=1);
+        basico.set_parameters('c', exact=True, expression='{[X]}+{[Y]}', status='assignment')
+        params31 = basico.as_dict(basico.get_parameters(type='assignment'))
+        basico.set_parameters('c', exact=True, expression=' {[X]} + {[Y]}', status='assignment')
+        params3 = basico.as_dict(basico.get_parameters(type='assignment'))
+        basico.set_parameters('c', exact=True, expression=' [X] + [Y]', status='assignment')
+        params4 = basico.as_dict(basico.get_parameters(type='assignment'))
+        params = basico.as_dict(basico.get_parameters(type='assignment'))
+
+
+        basico.remove_datamodel(dm);
+
+    def test_multiple_elements_with_same_name(self):
+        dm = basico.new_model(name='Test Model');
+
+        basico.add_reaction('R1', ' -> "Time"')
+        basico.add_compartment('Time')
+        basico.add_parameter('Time', initial_value=1)
+
+        result = basico.run_time_course()
+        self.assertListEqual(result.columns.tolist(), ['Time', 'Time'])
+
+        result2 = basico.run_time_course_with_output(['Time', '[Time]'])
+        self.assertListEqual(result2.columns.tolist(), ['Time', '[Time]'])
+
+        # test using collect:
+        data = basico.model_info.collect_data(names=['Time', '[Time]'])
+        self.assertTrue(data is not None)
+        self.assertListEqual(data.index.to_list(), ['Time', '[Time]'])
+
+
+        names = basico.model_info._get_name_map(dm)
+        self.assertGreater(len(names['Time']), 1)
+
+        did_rename = basico.ensure_unique_names()
+        self.assertTrue(did_rename)
+
+        names2 = basico.model_info._get_name_map(dm)
+        self.assertEqual(len(names2['Time']), 1)
+        
+
+        basico.remove_datamodel(dm);
+
 if __name__ == "__main__":
     unittest.main()
